@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2021 libsixel developers. See `AUTHORS`.
  * Copyright (c) 2014-2018 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
@@ -20,32 +21,19 @@
  */
 
 #include "config.h"
-#include "malloc_stub.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
 
-#if HAVE_UNISTD_H
 # include <unistd.h>
-#elif HAVE_SYS_UNISTD_H
 # include <sys/unistd.h>
-#endif
-#if HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#endif
-#if HAVE_GETOPT_H
 # include <getopt.h>
-#endif
-#if HAVE_INTTYPES_H
 # include <inttypes.h>
-#endif
-#if HAVE_SIGNAL_H
 # include <signal.h>
-#elif HAVE_SYS_SIGNAL_H
 # include <sys/signal.h>
-#endif
 
 #include <sixel.h>
 
@@ -303,6 +291,7 @@ void show_help(void)
             "                           pass-through sequence\n"
             "-D, --pipe-mode            [[deprecated]] read source images from\n"
             "                           stdin continuously\n"
+            "-O, --ormode               output ormode sixel image\n"
             "-v, --verbose              show debugging info\n"
             "-V, --version              show version and license info\n"
             "-H, --help                 show this help\n"
@@ -323,7 +312,6 @@ void show_help(void)
             );
 }
 
-#if HAVE_SIGNAL
 
 static int signaled = 0;
 
@@ -333,7 +321,6 @@ signal_handler(int sig)
     signaled = sig;
 }
 
-#endif
 
 int
 main(int argc, char *argv[])
@@ -341,14 +328,14 @@ main(int argc, char *argv[])
     SIXELSTATUS status = SIXEL_FALSE;
     int n;
     sixel_encoder_t *encoder = NULL;
-#if HAVE_GETOPT_LONG
+#ifdef HAVE_GETOPT_LONG
     int long_opt;
     int option_index;
 #endif  /* HAVE_GETOPT_LONG */
-    char const *optstring = "o:78Rp:m:eb:Id:f:s:c:w:h:r:q:kil:t:ugvSn:PE:B:C:DVH";
-#if HAVE_GETOPT_LONG
+    char const *optstring = "o:78ORp:m:eb:Id:f:s:c:w:h:r:q:kil:t:ugvSn:PE:B:C:DVH";
+#ifdef HAVE_GETOPT_LONG
     struct option long_options[] = {
-        {"outfile",          required_argument,  &long_opt, 'o'},
+        {"outfile",          no_argument,        &long_opt, 'o'},
         {"7bit-mode",        no_argument,        &long_opt, '7'},
         {"8bit-mode",        no_argument,        &long_opt, '8'},
         {"gri-limit",        no_argument,        &long_opt, 'R'},
@@ -379,6 +366,7 @@ main(int argc, char *argv[])
         {"bgcolor",          required_argument,  &long_opt, 'B'},
         {"complexion-score", required_argument,  &long_opt, 'C'},
         {"pipe-mode",        no_argument,        &long_opt, 'D'}, /* deprecated */
+        {"ormode",           no_argument,        &long_opt, 'O'},
         {"version",          no_argument,        &long_opt, 'V'},
         {"help",             no_argument,        &long_opt, 'H'},
         {0, 0, 0, 0}
@@ -392,7 +380,7 @@ main(int argc, char *argv[])
 
     for (;;) {
 
-#if HAVE_GETOPT_LONG
+#ifdef HAVE_GETOPT_LONG
         n = getopt_long(argc, argv, optstring,
                         long_options, &option_index);
 #else
@@ -402,7 +390,7 @@ main(int argc, char *argv[])
         if (n == (-1)) {
             break;
         }
-#if HAVE_GETOPT_LONG
+#ifdef HAVE_GETOPT_LONG
         if (n == 0) {
             n = long_opt;
         }
@@ -427,23 +415,13 @@ main(int argc, char *argv[])
     }
 
     /* set signal handler to handle SIGINT/SIGTERM/SIGHUP */
-#if HAVE_SIGNAL
-# if HAVE_DECL_SIGINT
     signal(SIGINT, signal_handler);
-# endif
-# if HAVE_DECL_SIGTERM
     signal(SIGTERM, signal_handler);
-# endif
-# if HAVE_DECL_SIGHUP
     signal(SIGHUP, signal_handler);
-# endif
     status = sixel_encoder_set_cancel_flag(encoder, &signaled);
     if (SIXEL_FAILED(status)) {
         goto error;
     }
-#else
-    (void) signal_handler;
-#endif
 
     if (optind == argc) {
         status = sixel_encoder_encode(encoder, NULL);
@@ -465,7 +443,7 @@ main(int argc, char *argv[])
 
 argerr:
     fprintf(stderr,
-            "usage: img2sixel [-78eIkiugvSPDVH] [-p colors] [-m file] [-d diffusiontype]\n"
+            "usage: img2sixel [-78eIkiugvSPDOVH] [-p colors] [-m file] [-d diffusiontype]\n"
             "                 [-f findtype] [-s selecttype] [-c geometory] [-w width]\n"
             "                 [-h height] [-r resamplingtype] [-q quality] [-l loopmode]\n"
             "                 [-t palettetype] [-n macronumber] [-C score] [-b palette]\n"
