@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2021 libsixel developers. See `AUTHORS`.
  * Copyright (c) 2014-2020 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
@@ -21,26 +22,15 @@
 
 #include "config.h"
 
-/* STDC_HEADERS */
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-
-#if HAVE_MATH_H
+# include <string.h>
+# include <stdlib.h>
+# include <stdio.h>
 # include <math.h>
-#endif  /* HAVE_MATH_H */
-#if HAVE_LIMITS_H
 # include <limits.h>
-#endif  /* HAVE_LIMITS_H */
-#if HAVE_INTTYPES_H
 # include <inttypes.h>
-#endif  /* HAVE_INTTYPES_H */
 
 #include "frame.h"
 
-#if !defined(HAVE_MEMMOVE)
-# define memmove(d, s, n) (bcopy ((s), (d), (n)))
-#endif
 
 /* constructor of frame object */
 SIXELAPI SIXELSTATUS
@@ -553,11 +543,6 @@ sixel_frame_resize(
         goto end;
     }
 
-    if (width == frame->width && height == frame->height) {
-        /* nothing to do */
-        goto out;
-    }
-
     status = sixel_frame_convert_to_rgb888(frame);
     if (SIXEL_FAILED(status)) {
         goto end;
@@ -590,7 +575,6 @@ sixel_frame_resize(
     frame->width = width;
     frame->height = height;
 
-out:
     status = SIXEL_OK;
 
 end:
@@ -764,14 +748,10 @@ test1(void)
     sixel_frame_t *frame = NULL;
     int nret = EXIT_FAILURE;
 
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic push
 #  pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
     frame = sixel_frame_create();
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic pop
-#endif
     if (frame == NULL) {
         goto error;
     }
@@ -801,14 +781,10 @@ test2(void)
 
     memset(bgcolor, 0x10, 3);
 
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic push
 #  pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
     frame = sixel_frame_create();
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic pop
-#endif
 
     if (frame == NULL) {
         goto error;
@@ -867,14 +843,10 @@ test3(void)
     pixels[2] = 0x97;
     pixels[3] = 0x32;
 
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic push
 #  pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
     frame = sixel_frame_create();
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic pop
-#endif
     if (frame == NULL) {
         goto error;
     }
@@ -932,14 +904,10 @@ test4(void)
     pixels[2] = 0x97;
     pixels[3] = 0x32;
 
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic push
 #  pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
     frame = sixel_frame_create();
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic pop
-#endif
     if (frame == NULL) {
         goto error;
     }
@@ -999,14 +967,10 @@ test5(void)
 
     pixels[0] = 0;
 
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic push
 #  pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
     frame = sixel_frame_create();
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic pop
-#endif
     if (frame == NULL) {
         goto error;
     }
@@ -1066,14 +1030,10 @@ test6(void)
 
     pixels[0] = 0;
 
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic push
 #  pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
     frame = sixel_frame_create();
-#if HAVE_DIAGNOSTIC_DEPRECATED_DECLARATIONS
 #  pragma GCC diagnostic pop
-#endif
     if (frame == NULL) {
         goto error;
     }
