@@ -5,6 +5,7 @@
  * Hayaki Saito <saitoha@me.com> modified this and re-licensed
  * it under the MIT license.
  *
+ * Copyright (c) 2021 libsixel developers. See `AUTHORS`.
  * Copyright (c) 2014-2018 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
@@ -27,19 +28,11 @@
 
 #include "config.h"
 
-/* STDC_HEADERS */
-#include <stdio.h>
-#include <stdlib.h>
-
-#if HAVE_STRING_H
+# include <stdio.h>
+# include <stdlib.h>
 # include <string.h>
-#endif  /* HAVE_STRING_H */
-#if HAVE_CTYPE_H
 # include <ctype.h>
-#endif  /* HAVE_CTYPE_H */
-#if HAVE_ASSERT_H
 # include <assert.h>
-#endif  /* HAVE_ASSERT_H */
 
 #include "frame.h"
 #include "fromgif.h"
@@ -637,15 +630,17 @@ load_gif(
     if (status != SIXEL_OK) {
         goto end;
     }
-    g.out = (unsigned char *)sixel_allocator_malloc(allocator, (size_t)g.w * (size_t)g.h);
+    size_t bytes = (size_t)g.w * (size_t)g.h;
+    g.out = (unsigned char *)sixel_allocator_malloc(allocator, bytes);
     if (g.out == NULL) {
         sprintf(message,
                 "load_gif: sixel_allocator_malloc() failed. size=%zu.",
-                (size_t)g.max_x * (size_t)g.max_y);
+                bytes);
         sixel_helper_set_additional_message(message);
         status = SIXEL_BAD_ALLOCATION;
         goto end;
     }
+    memset(g.out, 0, bytes);
 
     frame->loop_count = 0;
 
