@@ -12,25 +12,18 @@
  *
  * Hayaki Saito <saitoha@me.com> modified this and re-licensed
  * it to the MIT license.
+ *
+ * Copyright (c) 2021 libsixel developers. See `AUTHORS`.
+ * Copyright (c) 2014-2020 Hayaki Saito
  */
 #include "config.h"
 
-/* STDC_HEADERS */
-#include <stdlib.h>
-#include <stdio.h>
-
-#if HAVE_CTYPE_H
+# include <stdlib.h>
+# include <stdio.h>
 # include <ctype.h>   /* isdigit */
-#endif  /* HAVE_CTYPE_H */
-#if HAVE_STRING_H
 # include <string.h>  /* memcpy */
-#endif  /* HAVE_STRING_H */
-#if HAVE_LIMITS_H
 # include <limits.h>
-#endif  /* HAVE_LIMITS_H */
-#if HAVE_INTTYPES_H
 # include <inttypes.h>
-#endif  /* HAVE_INTTYPES_H */
 
 #include <sixel.h>
 #include "output.h"
@@ -117,12 +110,12 @@ hls_to_rgb(int hue, int lum, int sat)
     }
 
     /* https://wikimedia.org/api/rest_v1/media/math/render/svg/17e876f7e3260ea7fed73f69e19c71eb715dd09d */
-    max = lum + sat * (1.0 - (lum > 50 ? (2 * (lum / 100.0) - 1.0): - (2 * (lum / 100.0) - 1.0))) / 2.0;
+    max = lum + sat * (1.0 - (lum > 50 ? (((lum << 2) / 100.0) - 1.0): - (2 * (lum / 100.0) - 1.0))) / 2.0;
 
     /* https://wikimedia.org/api/rest_v1/media/math/render/svg/f6721b57985ad83db3d5b800dc38c9980eedde1d */
-    min = lum - sat * (1.0 - (lum > 50 ? (2 * (lum / 100.0) - 1.0): - (2 * (lum / 100.0) - 1.0))) / 2.0;
+    min = lum - sat * (1.0 - (lum > 50 ? (((lum << 2) / 100.0) - 1.0): - (2 * (lum / 100.0) - 1.0))) / 2.0;
 
-    /* sixel hue color ring is roteted -120 degree from nowdays general one. */
+    /* sixel hue color ring is rotated -120 degree from one generally used today. */
     hue = (hue + 240) % 360;
 
     /* https://wikimedia.org/api/rest_v1/media/math/render/svg/937e8abdab308a22ff99de24d645ec9e70f1e384 */
@@ -158,9 +151,7 @@ hls_to_rgb(int hue, int lum, int sat)
         b = (min + (max - min) * ((360 - hue) / 60.0));
         break;
     default:
-#if HAVE___BUILTIN_UNREACHABLE
         __builtin_unreachable();
-#endif
         break;
     }
 
