@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2021 libsixel developers. See `AUTHORS`.
  * Copyright (c) 2014-2019 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
@@ -21,42 +22,27 @@
 
 #include "config.h"
 
-/* STDC_HEADERS */
-#include <stdio.h>
-#include <stdlib.h>
-
-#if HAVE_STRING_H
+# include <stdio.h>
+# include <stdlib.h>
 # include <string.h>
-#endif
-#if HAVE_UNISTD_H
 # include <unistd.h>
-#endif
-#if HAVE_ERRNO_H
 # include <errno.h>
-#endif
 #ifdef HAVE_GDK_PIXBUF2
-# if HAVE_DIAGNOSTIC_TYPEDEF_REDEFINITION
-#   pragma GCC diagnostic push
-#   pragma GCC diagnostic ignored "-Wtypedef-redefinition"
-# endif
+//#   pragma GCC diagnostic push
+//#   pragma GCC diagnostic ignored "-Wtypedef-redefinition"
 # include <gdk-pixbuf/gdk-pixbuf.h>
-# if HAVE_DIAGNOSTIC_TYPEDEF_REDEFINITION
-#   pragma GCC diagnostic pop
-# endif
+//#   pragma GCC diagnostic pop
 #endif
-#if HAVE_GD
+#ifdef HAVE_GD
 # include <gd.h>
 #endif
-#if HAVE_LIBPNG
+#ifdef HAVE_LIBPNG
 # include <png.h>
 #endif  /* HAVE_LIBPNG */
-#if HAVE_JPEG
+#ifdef HAVE_JPEG
 # include <jpeglib.h>
 #endif  /* HAVE_JPEG */
 
-#if !defined(HAVE_MEMCPY)
-# define memcpy(d, s, n) (bcopy ((s), (d), (n)))
-#endif
 
 #include "frame.h"
 #include <sixel.h>
@@ -95,54 +81,30 @@ stbi_free(void *p)
 #define STBI_NO_GIF
 #define STBI_NO_PNM
 
-#if HAVE_DIAGNOSTIC_SIGN_CONVERSION
+#ifdef HAVE_DIAGNOSTIC_SIGN_CONVERSION
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wsign-conversion"
 #endif
-#if HAVE_DIAGNOSTIC_STRICT_OVERFLOW
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wstrict-overflow"
-#endif
-#if HAVE_DIAGNOSTIC_SWITCH_DEFAULT
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wswitch-default"
-#endif
-#if HAVE_DIAGNOSTIC_SHADOW
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wshadow"
-#endif
-#if HAVE_DIAGNOSTIC_DOUBLE_PROMOTION
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wdouble-promotion"
-#endif
-# if HAVE_DIAGNOSTIC_UNUSED_FUNCTION
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wunused-function"
-#endif
-# if HAVE_DIAGNOSTIC_UNUSED_BUT_SET_VARIABLE
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wunused-but-set-variable"
-#endif
 #include "stb_image.h"
-#if HAVE_DIAGNOSTIC_UNUSED_BUT_SET_VARIABLE
 # pragma GCC diagnostic pop
-#endif
-#if HAVE_DIAGNOSTIC_UNUSED_FUNCTION
 # pragma GCC diagnostic pop
-#endif
-#if HAVE_DIAGNOSTIC_DOUBLE_PROMOTION
 # pragma GCC diagnostic pop
-#endif
-#if HAVE_DIAGNOSTIC_SHADOW
 # pragma GCC diagnostic pop
-#endif
-#if HAVE_DIAGNOSTIC_SWITCH_DEFAULT
 # pragma GCC diagnostic pop
-#endif
-#if HAVE_DIAGNOSTIC_STRICT_OVERFLOW
 # pragma GCC diagnostic pop
-#endif
-#if HAVE_DIAGNOSTIC_SIGN_CONVERSION
+#ifdef HAVE_DIAGNOSTIC_SIGN_CONVERSION
 # pragma GCC diagnostic pop
 #endif
 
@@ -184,11 +146,6 @@ load_jpeg(unsigned char **result,
     }
 
     *ppixelformat = SIXEL_PIXELFORMAT_RGB888;
-
-    if (cinfo.output_width > INT_MAX || cinfo.output_height > INT_MAX) {
-        status = SIXEL_BAD_INTEGER_OVERFLOW;
-        goto end;
-    }
     *pwidth = (int)cinfo.output_width;
     *pheight = (int)cinfo.output_height;
 
@@ -287,9 +244,7 @@ png_error_callback(png_structp png_ptr, png_const_charp error_message)
     (void) png_ptr;
 
     sixel_helper_set_additional_message(error_message);
-#if HAVE_SETJMP && HAVE_LONGJMP
     longjmp(jmpbuf, 1);
-#endif  /* HAVE_SETJMP && HAVE_LONGJMP */
 }
 
 
@@ -313,27 +268,21 @@ load_png(unsigned char      /* out */ **result,
     png_uint_32 png_status;
     png_structp png_ptr;
     png_infop info_ptr;
-#ifdef HAVE_DIAGNOSTIC_CLOBBERED
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wclobbered"
-#endif
     unsigned char **rows = NULL;
     png_color *png_palette = NULL;
     png_color_16 background;
     png_color_16p default_background;
-    png_uint_32 width;
-    png_uint_32 height;
     int i;
     int depth;
 
-#if HAVE_SETJMP && HAVE_LONGJMP
     if (setjmp(jmpbuf) != 0) {
         sixel_allocator_free(allocator, *result);
         *result = NULL;
         status = SIXEL_PNG_ERROR;
         goto cleanup;
     }
-#endif  /* HAVE_SETJMP && HAVE_LONGJMP */
 
     status = SIXEL_FALSE;
     *result = NULL;
@@ -355,14 +304,12 @@ load_png(unsigned char      /* out */ **result,
         goto cleanup;
     }
 
-#if HAVE_SETJMP
     if (setjmp(png_jmpbuf(png_ptr)) != 0) {
         sixel_allocator_free(allocator, *result);
         *result = NULL;
         status = SIXEL_PNG_ERROR;
         goto cleanup;
     }
-#endif  /* HAVE_SETJMP */
 
 
     info_ptr = png_create_info_struct(png_ptr);
@@ -378,18 +325,8 @@ load_png(unsigned char      /* out */ **result,
 
     png_set_read_fn(png_ptr,(png_voidp)&read_chunk, read_png);
     png_read_info(png_ptr, info_ptr);
-
-    width = png_get_image_width(png_ptr, info_ptr);
-    height = png_get_image_height(png_ptr, info_ptr);
-
-    if (width > INT_MAX || height > INT_MAX) {
-        status = SIXEL_BAD_INTEGER_OVERFLOW;
-        goto cleanup;
-    }
-
-    *psx = (int)width;
-    *psy = (int)height;
-
+    *psx = (int)png_get_image_width(png_ptr, info_ptr);
+    *psy = (int)png_get_image_height(png_ptr, info_ptr);
     bitdepth = png_get_bit_depth(png_ptr, info_ptr);
     if (bitdepth == 16) {
 #  if HAVE_DEBUG
@@ -662,9 +599,7 @@ cleanup:
 
     return status;
 }
-#ifdef HAVE_DIAGNOSTIC_CLOBBERED
 # pragma GCC diagnostic pop
-#endif
 
 # endif  /* HAVE_LIBPNG */
 
@@ -778,7 +713,7 @@ chunk_is_pnm(sixel_chunk_t const *chunk)
 }
 
 
-#if HAVE_LIBPNG
+#ifdef HAVE_LIBPNG
 /* detect whether given chunk is PNG stream */
 static int
 chunk_is_png(sixel_chunk_t const *chunk)
@@ -813,7 +748,7 @@ chunk_is_gif(sixel_chunk_t const *chunk)
 }
 
 
-#if HAVE_JPEG
+#ifdef HAVE_JPEG
 /* detect whether given chunk is JPEG stream */
 static int
 chunk_is_jpeg(sixel_chunk_t const *chunk)
@@ -857,10 +792,6 @@ load_with_builtin(
         if (SIXEL_FAILED(status)) {
             goto end;
         }
-        if (pchunk->size > INT_MAX) {
-            status = SIXEL_BAD_INTEGER_OVERFLOW;
-            goto end;
-        }
         status = load_sixel(&frame->pixels,
                             pchunk->buffer,
                             (int)pchunk->size,
@@ -879,10 +810,6 @@ load_with_builtin(
         if (SIXEL_FAILED(status)) {
             goto end;
         }
-        if (pchunk->size > INT_MAX) {
-            status = SIXEL_BAD_INTEGER_OVERFLOW;
-            goto end;
-        }
         /* pnm */
         status = load_pnm(pchunk->buffer,
                           (int)pchunk->size,
@@ -897,7 +824,7 @@ load_with_builtin(
             goto end;
         }
     }
-#if HAVE_JPEG
+#ifdef HAVE_JPEG
     else if (chunk_is_jpeg(pchunk)) {
         status = sixel_frame_new(&frame, pchunk->allocator);
         if (SIXEL_FAILED(status)) {
@@ -916,7 +843,7 @@ load_with_builtin(
         }
     }
 #endif  /* HAVE_JPEG */
-#if HAVE_LIBPNG
+#ifdef HAVE_LIBPNG
     else if (chunk_is_png(pchunk)) {
         status = sixel_frame_new(&frame, pchunk->allocator);
         if (SIXEL_FAILED(status)) {
@@ -941,10 +868,6 @@ load_with_builtin(
 #endif  /* HAVE_LIBPNG */
     else if (chunk_is_gif(pchunk)) {
         fnp.fn = fn_load;
-        if (pchunk->size > INT_MAX) {
-            status = SIXEL_BAD_INTEGER_OVERFLOW;
-            goto end;
-        }
         status = load_gif(pchunk->buffer,
                           (int)pchunk->size,
                           bgcolor,
@@ -968,10 +891,6 @@ load_with_builtin(
             goto end;
         }
         stbi_allocator = pchunk->allocator;
-        if (pchunk->size > INT_MAX) {
-            status = SIXEL_BAD_INTEGER_OVERFLOW;
-            goto end;
-        }
         stbi__start_mem(&s, pchunk->buffer, (int)pchunk->size);
         frame->pixels = stbi__load_and_postprocess_8bit(&s, &frame->width, &frame->height, &depth, 3);
         if (!frame->pixels) {
@@ -1285,45 +1204,45 @@ load_with_gd(
 
     switch (detect_file_format(pchunk->size, pchunk->buffer)) {
 #if 0
-# if HAVE_DECL_GDIMAGECREATEFROMGIFPTR
+#ifdef HAVE_DECL_GDIMAGECREATEFROMGIFPTR
         case SIXEL_FORMAT_GIF:
             im = gdImageCreateFromGifPtr(pchunk->size, pchunk->buffer);
             break;
 # endif  /* HAVE_DECL_GDIMAGECREATEFROMGIFPTR */
 #endif
-#if HAVE_DECL_GDIMAGECREATEFROMPNGPTR
+#ifdef HAVE_DECL_GDIMAGECREATEFROMPNGPTR
         case SIXEL_FORMAT_PNG:
             im = gdImageCreateFromPngPtr(pchunk->size, pchunk->buffer);
             break;
 #endif  /* HAVE_DECL_GDIMAGECREATEFROMPNGPTR */
-#if HAVE_DECL_GDIMAGECREATEFROMBMPPTR
+#ifdef HAVE_DECL_GDIMAGECREATEFROMBMPPTR
         case SIXEL_FORMAT_BMP:
             im = gdImageCreateFromBmpPtr(pchunk->size, pchunk->buffer);
             break;
 #endif  /* HAVE_DECL_GDIMAGECREATEFROMBMPPTR */
         case SIXEL_FORMAT_JPG:
-#if HAVE_DECL_GDIMAGECREATEFROMJPEGPTREX
+#ifdef HAVE_DECL_GDIMAGECREATEFROMJPEGPTREX
             im = gdImageCreateFromJpegPtrEx(pchunk->size, pchunk->buffer, 1);
 #elif HAVE_DECL_GDIMAGECREATEFROMJPEGPTR
             im = gdImageCreateFromJpegPtr(pchunk->size, pchunk->buffer);
 #endif  /* HAVE_DECL_GDIMAGECREATEFROMJPEGPTREX */
             break;
-#if HAVE_DECL_GDIMAGECREATEFROMTGAPTR
+#ifdef HAVE_DECL_GDIMAGECREATEFROMTGAPTR
         case SIXEL_FORMAT_TGA:
             im = gdImageCreateFromTgaPtr(pchunk->size, pchunk->buffer);
             break;
 #endif  /* HAVE_DECL_GDIMAGECREATEFROMTGAPTR */
-#if HAVE_DECL_GDIMAGECREATEFROMWBMPPTR
+#ifdef HAVE_DECL_GDIMAGECREATEFROMWBMPPTR
         case SIXEL_FORMAT_WBMP:
             im = gdImageCreateFromWBMPPtr(pchunk->size, pchunk->buffer);
             break;
 #endif  /* HAVE_DECL_GDIMAGECREATEFROMWBMPPTR */
-#if HAVE_DECL_GDIMAGECREATEFROMTIFFPTR
+#ifdef HAVE_DECL_GDIMAGECREATEFROMTIFFPTR
         case SIXEL_FORMAT_TIFF:
             im = gdImageCreateFromTiffPtr(pchunk->size, pchunk->buffer);
             break;
 #endif  /* HAVE_DECL_GDIMAGECREATEFROMTIFFPTR */
-#if HAVE_DECL_GDIMAGECREATEFROMGD2PTR
+#ifdef HAVE_DECL_GDIMAGECREATEFROMGD2PTR
         case SIXEL_FORMAT_GD2:
             im = gdImageCreateFromGd2Ptr(pchunk->size, pchunk->buffer);
             break;
@@ -1342,7 +1261,7 @@ load_with_gd(
     }
 
     if (!gdImageTrueColor(im)) {
-#if HAVE_DECL_GDIMAGEPALETTETOTRUECOLOR
+#ifdef HAVE_DECL_GDIMAGEPALETTETOTRUECOLOR
         if (!gdImagePaletteToTrueColor(im)) {
             status = SIXEL_GD_ERROR;
             /* TODO: retrieve error detail */
@@ -1446,7 +1365,7 @@ sixel_helper_load_image_file(
                                      context);
     }
 #endif  /* HAVE_GDK_PIXBUF2 */
-#if HAVE_GD
+#ifdef HAVE_GD
     if (SIXEL_FAILED(status)) {
         status = load_with_gd(pchunk,
                               fstatic,
@@ -1479,7 +1398,7 @@ end:
 }
 
 
-#if HAVE_TESTS
+#ifdef HAVE_TESTS
 static int
 test1(void)
 {
