@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2021 libsixel developers. See `AUTHORS`.
  * Copyright (c) 2014-2019 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
@@ -21,13 +22,9 @@
 
 #include "config.h"
 
-/* STDC_HEADERS */
-#include <stdio.h>
-#include <stdlib.h>
-
-#if HAVE_MEMORY_H
+# include <stdio.h>
+# include <stdlib.h>
 # include <memory.h>
-#endif  /* HAVE_MEMORY_H */
 
 #include <sixel.h>
 
