@@ -22,6 +22,7 @@
  *
  * ******************************************************************************
  *
+ * Copyright (c) 2021 libsixel developers. See `AUTHORS`.
  * Copyright (c) 2014-2018 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
@@ -46,22 +47,12 @@
 
 #include "config.h"
 
-/* STDC_HEADERS */
-#include <stdlib.h>
-#include <stdio.h>
-
-#if HAVE_STRING_H
+# include <stdlib.h>
+# include <stdio.h>
 # include <string.h>
-#endif  /* HAVE_STRING_H */
-#if HAVE_MATH_H
 #include <math.h>
-#endif  /* HAVE_MATH_H */
-#if HAVE_LIMITS_H
 # include <limits.h>
-#endif  /* HAVE_MATH_H */
-#if HAVE_INTTYPES_H
 # include <inttypes.h>
-#endif  /* HAVE_MATH_H */
 
 #include "quant.h"
 
@@ -700,17 +691,23 @@ computeHistogram(unsigned char const    /* in */  *data,
     switch (qualityMode) {
     case SIXEL_QUALITY_LOW:
         max_sample = 18383;
+        step = length / depth / max_sample * depth;
         break;
     case SIXEL_QUALITY_HIGH:
-        max_sample = 1118383;
+        max_sample = 18383;
+        step = length / depth / max_sample * depth;
         break;
     case SIXEL_QUALITY_FULL:
     default:
         max_sample = 4003079;
+        step = length / depth / max_sample * depth;
         break;
     }
 
-    step = length / depth / max_sample * depth;
+    if (length < max_sample * depth) {
+        step = 6 * depth;
+    }
+
     if (step <= 0) {
         step = depth;
     }
@@ -856,15 +853,20 @@ end:
 
 /* diffuse error energy to surround pixels */
 static void
-error_diffuse(unsigned char /* in */    *data,      /* base address of pixel buffer */
-              int           /* in */    pos,        /* address of the destination pixel */
-              int           /* in */    depth,      /* color depth in bytes */
-              int           /* in */    error,      /* error energy */
-              int           /* in */    numerator,  /* numerator of diffusion coefficient */
-              int           /* in */    denominator /* denominator of diffusion coefficient */)
+error_diffuse(unsigned char /* in */    *data,       /* base address of pixel buffer */
+              int           /* in */    pos,         /* address of the destination pixel */
+              int           /* in */    depth,       /* color depth in bytes */
+              int           /* in */    error,       /* error energy */
+              int           /* in */    numerator,   /* numerator of diffusion coefficient */
+              int           /* in */    denominator ,/* denominator of diffusion coefficient */
+              int           /* in */    area         /* area, in pixels */)
 {
     int c;
 
+    if (pos < 0 || pos >= area) {
+        return;
+    }
+
     data += pos * depth;
 
     c = *data + error * numerator / denominator;
@@ -906,13 +908,13 @@ diffuse_fs(unsigned char *data, int width, int height,
      */
     if (x < width - 1 && y < height - 1) {
         /* add error to the right cell */
-        error_diffuse(data, pos + width * 0 + 1, depth, error, 7, 16);
+        error_diffuse(data, pos + width * 0 + 1, depth, error, 7, 16, width * height);
         /* add error to the left-bottom cell */
-        error_diffuse(data, pos + width * 1 - 1, depth, error, 3, 16);
+        error_diffuse(data, pos + width * 1 - 1, depth, error, 3, 16, width * height);
         /* add error to the bottom cell */
-        error_diffuse(data, pos + width * 1 + 0, depth, error, 5, 16);
+        error_diffuse(data, pos + width * 1 + 0, depth, error, 5, 16, width * height);
         /* add error to the right-bottom cell */
-        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 16);
+        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 16, width * height);
     }
 }
 
@@ -932,17 +934,17 @@ diffuse_atkinson(unsigned char *data, int width, int height,
      */
     if (y < height - 2) {
         /* add error to the right cell */
-        error_diffuse(data, pos + width * 0 + 1, depth, error, 1, 8);
+        error_diffuse(data, pos + width * 0 + 1, depth, error, 1, 8, width * height);
         /* add error to the 2th right cell */
-        error_diffuse(data, pos + width * 0 + 2, depth, error, 1, 8);
+        error_diffuse(data, pos + width * 0 + 2, depth, error, 1, 8, width * height);
         /* add error to the left-bottom cell */
-        error_diffuse(data, pos + width * 1 - 1, depth, error, 1, 8);
+        error_diffuse(data, pos + width * 1 - 1, depth, error, 1, 8, width * height);
         /* add error to the bottom cell */
-        error_diffuse(data, pos + width * 1 + 0, depth, error, 1, 8);
+        error_diffuse(data, pos + width * 1 + 0, depth, error, 1, 8, width * height);
         /* add error to the right-bottom cell */
-        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 8);
+        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 8, width * height);
         /* add error to the 2th bottom cell */
-        error_diffuse(data, pos + width * 2 + 0, depth, error, 1, 8);
+        error_diffuse(data, pos + width * 2 + 0, depth, error, 1, 8, width * height);
     }
 }
 
@@ -961,18 +963,18 @@ diffuse_jajuni(unsigned char *data, int width, int height,
      *  1/48    3/48    5/48    3/48    1/48
      */
     if (pos < (height - 2) * width - 2) {
-        error_diffuse(data, pos + width * 0 + 1, depth, error, 7, 48);
-        error_diffuse(data, pos + width * 0 + 2, depth, error, 5, 48);
-        error_diffuse(data, pos + width * 1 - 2, depth, error, 3, 48);
-        error_diffuse(data, pos + width * 1 - 1, depth, error, 5, 48);
-        error_diffuse(data, pos + width * 1 + 0, depth, error, 7, 48);
-        error_diffuse(data, pos + width * 1 + 1, depth, error, 5, 48);
-        error_diffuse(data, pos + width * 1 + 2, depth, error, 3, 48);
-        error_diffuse(data, pos + width * 2 - 2, depth, error, 1, 48);
-        error_diffuse(data, pos + width * 2 - 1, depth, error, 3, 48);
-        error_diffuse(data, pos + width * 2 + 0, depth, error, 5, 48);
-        error_diffuse(data, pos + width * 2 + 1, depth, error, 3, 48);
-        error_diffuse(data, pos + width * 2 + 2, depth, error, 1, 48);
+        error_diffuse(data, pos + width * 0 + 1, depth, error, 7, 48, width * height);
+        error_diffuse(data, pos + width * 0 + 2, depth, error, 5, 48, width * height);
+        error_diffuse(data, pos + width * 1 - 2, depth, error, 3, 48, width * height);
+        error_diffuse(data, pos + width * 1 - 1, depth, error, 5, 48, width * height);
+        error_diffuse(data, pos + width * 1 + 0, depth, error, 7, 48, width * height);
+        error_diffuse(data, pos + width * 1 + 1, depth, error, 5, 48, width * height);
+        error_diffuse(data, pos + width * 1 + 2, depth, error, 3, 48, width * height);
+        error_diffuse(data, pos + width * 2 - 2, depth, error, 1, 48, width * height);
+        error_diffuse(data, pos + width * 2 - 1, depth, error, 3, 48, width * height);
+        error_diffuse(data, pos + width * 2 + 0, depth, error, 5, 48, width * height);
+        error_diffuse(data, pos + width * 2 + 1, depth, error, 3, 48, width * height);
+        error_diffuse(data, pos + width * 2 + 2, depth, error, 1, 48, width * height);
     }
 }
 
@@ -991,18 +993,18 @@ diffuse_stucki(unsigned char *data, int width, int height,
      *  1/48    2/48    4/48    2/48    1/48
      */
     if (pos < (height - 2) * width - 2) {
-        error_diffuse(data, pos + width * 0 + 1, depth, error, 1, 6);
-        error_diffuse(data, pos + width * 0 + 2, depth, error, 1, 12);
-        error_diffuse(data, pos + width * 1 - 2, depth, error, 1, 24);
-        error_diffuse(data, pos + width * 1 - 1, depth, error, 1, 12);
-        error_diffuse(data, pos + width * 1 + 0, depth, error, 1, 6);
-        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 12);
-        error_diffuse(data, pos + width * 1 + 2, depth, error, 1, 24);
-        error_diffuse(data, pos + width * 2 - 2, depth, error, 1, 48);
-        error_diffuse(data, pos + width * 2 - 1, depth, error, 1, 24);
-        error_diffuse(data, pos + width * 2 + 0, depth, error, 1, 12);
-        error_diffuse(data, pos + width * 2 + 1, depth, error, 1, 24);
-        error_diffuse(data, pos + width * 2 + 2, depth, error, 1, 48);
+        error_diffuse(data, pos + width * 0 + 1, depth, error, 1, 6, width * height);
+        error_diffuse(data, pos + width * 0 + 2, depth, error, 1, 12, width * height);
+        error_diffuse(data, pos + width * 1 - 2, depth, error, 1, 24, width * height);
+        error_diffuse(data, pos + width * 1 - 1, depth, error, 1, 12, width * height);
+        error_diffuse(data, pos + width * 1 + 0, depth, error, 1, 6, width * height);
+        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 12, width * height);
+        error_diffuse(data, pos + width * 1 + 2, depth, error, 1, 24, width * height);
+        error_diffuse(data, pos + width * 2 - 2, depth, error, 1, 48, width * height);
+        error_diffuse(data, pos + width * 2 - 1, depth, error, 1, 24, width * height);
+        error_diffuse(data, pos + width * 2 + 0, depth, error, 1, 12, width * height);
+        error_diffuse(data, pos + width * 2 + 1, depth, error, 1, 24, width * height);
+        error_diffuse(data, pos + width * 2 + 2, depth, error, 1, 48, width * height);
     }
 }
 
@@ -1020,13 +1022,13 @@ diffuse_burkes(unsigned char *data, int width, int height,
      *  1/16    2/16    4/16    2/16    1/16
      */
     if (pos < (height - 1) * width - 2) {
-        error_diffuse(data, pos + width * 0 + 1, depth, error, 1, 4);
-        error_diffuse(data, pos + width * 0 + 2, depth, error, 1, 8);
-        error_diffuse(data, pos + width * 1 - 2, depth, error, 1, 16);
-        error_diffuse(data, pos + width * 1 - 1, depth, error, 1, 8);
-        error_diffuse(data, pos + width * 1 + 0, depth, error, 1, 4);
-        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 8);
-        error_diffuse(data, pos + width * 1 + 2, depth, error, 1, 16);
+        error_diffuse(data, pos + width * 0 + 1, depth, error, 1, 4, width * height);
+        error_diffuse(data, pos + width * 0 + 2, depth, error, 1, 8, width * height);
+        error_diffuse(data, pos + width * 1 - 2, depth, error, 1, 16, width * height);
+        error_diffuse(data, pos + width * 1 - 1, depth, error, 1, 8, width * height);
+        error_diffuse(data, pos + width * 1 + 0, depth, error, 1, 4, width * height);
+        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 8, width * height);
+        error_diffuse(data, pos + width * 1 + 2, depth, error, 1, 16, width * height);
     }
 }
 
@@ -1283,13 +1285,6 @@ sixel_quant_apply_palette(
         goto end;
     }
 
-    /* NOTE: diffuse_jajuni, diffuse_stucki, and diffuse_burkes reference at
-     * minimum the position pos + width * 1 - 2, so width must be at least 2
-     * to avoid underflow.
-     * On the other hand, diffuse_fs and diffuse_atkinson
-     * reference pos + width * 1 - 1, but since these functions are only called
-     * when width >= 1, they do not cause underflow.
-     */
     if (depth != 3) {
         f_diffuse = diffuse_none;
     } else {
@@ -1304,16 +1299,13 @@ sixel_quant_apply_palette(
             f_diffuse = diffuse_fs;
             break;
         case SIXEL_DIFFUSE_JAJUNI:
-            /* fallback to diffuse_none if width < 2 */
-            f_diffuse = width >= 2 ? diffuse_jajuni: diffuse_none;
+            f_diffuse = diffuse_jajuni;
             break;
         case SIXEL_DIFFUSE_STUCKI:
-            /* fallback to diffuse_none if width < 2 */
-            f_diffuse = width >= 2 ? diffuse_stucki: diffuse_none;
+            f_diffuse = diffuse_stucki;
             break;
         case SIXEL_DIFFUSE_BURKES:
-            /* fallback to diffuse_none if width < 2 */
-            f_diffuse = width >= 2 ? diffuse_burkes: diffuse_none;
+            f_diffuse = diffuse_burkes;
             break;
         case SIXEL_DIFFUSE_A_DITHER:
             f_diffuse = diffuse_none;
