@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2021 libsixel developers. See `AUTHORS`.
  * Copyright (c) 2014-2016 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
@@ -21,13 +22,9 @@
 
 #include "config.h"
 
-/* STDC_HEADERS */
-#include <stdlib.h>
-
-#if HAVE_MATH_H
+# include <stdlib.h>
 # define _USE_MATH_DEFINES  /* for MSVC */
 # include <math.h>
-#endif  /* HAVE_MATH_H */
 #ifndef M_PI
 # define M_PI 3.14159265358979323846
 #endif
@@ -207,8 +204,8 @@ scale_without_resampling(
 
     for (h = 0; h < dsth; h++) {
         for (w = 0; w < dstw; w++) {
-            x = (long)w * srcw / dstw;
-            y = (long)h * srch / dsth;
+            x = w * srcw / dstw;
+            y = h * srch / dsth;
             for (i = 0; i < depth; i++) {
                 pos = (y * srcw + x) * depth + i;
                 dst[(h * dstw + w) * depth + i] = src[pos];
