@@ -16,22 +16,16 @@
  * Araki Ken added high-color encoding mode(sixel_encode_highcolor)
  * extension.
  *
+ * Copyright (c) 2021 libsixel developers. See `AUTHORS`.
+ * Copyright (c) 2014-2020 kmiya@culti, Araki Ken, Hayaki Saito
  */
 #include "config.h"
 
-/* STDC_HEADERS */
-#include <stdio.h>
-#include <stdlib.h>
-
-#if HAVE_STRING_H
+# include <stdio.h>
+# include <stdlib.h>
 # include <string.h>
-#endif  /* HAVE_STRING_H */
-#if HAVE_LIMITS_H
 # include <limits.h>
-#endif  /* HAVE_LIMITS_H */
-#if HAVE_INTTYPES_H
 # include <inttypes.h>
-#endif  /* HAVE_INTTYPES_H */
 
 #include <sixel.h>
 #include "output.h"
@@ -116,7 +110,6 @@ sixel_puts(unsigned char *buffer, char const *value, int size)
 }
 
 
-#if HAVE_LDIV
 static int
 sixel_putnum_impl(char *buffer, long value, int pos)
 {
@@ -129,7 +122,6 @@ sixel_putnum_impl(char *buffer, long value, int pos)
     *(buffer + pos) = '0' + r.rem;
     return pos + 1;
 }
-#endif  /* HAVE_LDIV */
 
 
 static int
@@ -137,11 +129,7 @@ sixel_putnum(char *buffer, int value)
 {
     int pos;
 
-#if HAVE_LDIV
     pos = sixel_putnum_impl(buffer, value, 0);
-#else
-    pos = sprintf(buffer, "%d", value);
-#endif  /* HAVE_LDIV */
 
     return pos;
 }
@@ -279,16 +267,20 @@ sixel_put_node(
     }
 
     for (; *x < np->sx; ++*x) {
-        status = sixel_put_pixel(output, 0);
-        if (SIXEL_FAILED(status)) {
-            goto end;
+        if (*x != keycolor) {
+            status = sixel_put_pixel(output, 0);
+            if (SIXEL_FAILED(status)) {
+                goto end;
+            }
         }
     }
 
     for (; *x < np->mx; ++*x) {
-        status = sixel_put_pixel(output, np->map[*x]);
-        if (SIXEL_FAILED(status)) {
-            goto end;
+        if (*x != keycolor) {
+            status = sixel_put_pixel(output, np->map[*x]);
+            if (SIXEL_FAILED(status)) {
+                goto end;
+            }
         }
     }
 
@@ -311,6 +303,11 @@ sixel_encode_header(int width, int height, sixel_output_t *output)
     int pcount = 3;
     int use_raster_attributes = 1;
 
+    if (output->ormode) {
+        p[0] = 7;
+        p[1] = 5;
+    }
+
     output->pos = 0;
 
     if (!output->skip_dcs_envelope) {
@@ -504,7 +501,6 @@ sixel_encode_body(
     int sx;
     int mx;
     int len;
-    int pix;
     char *map = NULL;
     int check_integer_overflow;
     sixel_node_t *np, *tp, top;
@@ -546,6 +542,8 @@ sixel_encode_body(
     }
 
     for (y = i = 0; y < height; y++) {
+        int pix;
+
         if (output->encode_policy != SIXEL_ENCODEPOLICY_SIZE) {
             fillable = 0;
         } else if (palstate) {
@@ -776,6 +774,109 @@ sixel_encode_footer(sixel_output_t *output)
     return status;
 }
 
+static SIXELSTATUS
+sixel_encode_body_ormode(
+    uint8_t             /* in */ *pixels,
+    int                 /* in */ width,
+    int                 /* in */ height,
+    unsigned char       /* in */ *palette,
+    int                 /* in */ ncolors,
+    int                 /* in */ keycolor,
+    sixel_output_t      /* in */ *output)
+{
+    SIXELSTATUS status;
+    int n;
+    int nplanes;
+    uint8_t *buf = pixels;
+    uint8_t *buf_p;
+    int x;
+    int cur_h;
+    int nwrite;
+    int plane;
+
+    for (n = 0; n < ncolors; n++) {
+        status = output_rgb_palette_definition(output, palette, n, keycolor);
+        if (SIXEL_FAILED(status)) {
+            return status;
+        }
+    }
+
+    for (nplanes = 8; nplanes > 1; nplanes--) {
+        if (ncolors > (1 << (nplanes - 1))) {
+            break;
+        }
+    }
+
+    for (cur_h = 6; cur_h <= height; cur_h += 6) {
+        for (plane = 0; plane < nplanes; plane++) {
+            sixel_putc(output->buffer + output->pos, '#');
+            sixel_advance(output, 1);
+            nwrite = sixel_putnum((char *)output->buffer + output->pos, 1 << plane);
+            sixel_advance(output, nwrite);
+
+            buf_p = buf;
+            for (x = 0; x < width; x++, buf_p++) {
+                sixel_put_pixel(output,
+                                ((buf_p[0] >> plane) & 0x1) |
+                                (((buf_p[width] >> plane) << 1) & 0x2) |
+                                (((buf_p[width * 2] >> plane) << 2) & 0x4) |
+                                (((buf_p[width * 3] >> plane) << 3) & 0x8) |
+                                (((buf_p[width * 4] >> plane) << 4) & 0x10) |
+                                (((buf_p[width * 5] >> plane) << 5) & 0x20));
+            }
+            status = sixel_put_flash(output);
+            if (SIXEL_FAILED(status)) {
+                return status;
+            }
+            sixel_putc(output->buffer + output->pos, '$');
+            sixel_advance(output, 1);
+        }
+        sixel_putc(output->buffer + output->pos, '-');
+        sixel_advance(output, 1);
+        buf += (width * 6);
+	}
+
+    if (cur_h > height) {
+        for (plane = 0; plane < nplanes; plane++) {
+            sixel_putc(output->buffer + output->pos, '#');
+            sixel_advance(output, 1);
+            nwrite = sixel_putnum((char *)output->buffer + output->pos, 1 << plane);
+            sixel_advance(output, nwrite);
+
+            buf_p = buf;
+            for (x = 0; x < width; x++) {
+                int pix = ((buf_p[0] >> plane) & 0x1);
+
+                switch(cur_h - height) {
+                    case 1:
+                        pix |= (((buf_p[width * 4] >> plane) << 4) & 0x10);
+                        /* Fall through */
+                    case 2:
+                        pix |= (((buf_p[width * 3] >> plane) << 3) & 0x8);
+                        /* Fall through */
+                    case 3:
+                        pix |= (((buf_p[width * 2] >> plane) << 2) & 0x4);
+                        /* Fall through */
+                    case 4:
+                    default:
+                        pix |= (((buf_p[width] >> plane) << 1) & 0x2);
+                        /* Fall through */
+                }
+                sixel_put_pixel(output, pix);
+            }
+            status = sixel_put_flash(output);
+            if (SIXEL_FAILED(status)) {
+                return status;
+            }
+
+            sixel_putc(output->buffer + output->pos, '$');
+            sixel_advance(output, 1);
+        }
+    }
+
+    return 0;
+}
+
 
 static SIXELSTATUS
 sixel_encode_dither(
@@ -838,16 +939,27 @@ sixel_encode_dither(
         goto end;
     }
 
-    status = sixel_encode_body(input_pixels,
-                               width,
-                               height,
-                               dither->palette,
-                               dither->ncolors,
-                               dither->keycolor,
-                               dither->bodyonly,
-                               output,
-                               NULL,
-                               dither->allocator);
+    if (output->ormode) {
+        status = sixel_encode_body_ormode(input_pixels,
+                                          width,
+                                          height,
+                                          dither->palette,
+                                          dither->ncolors,
+                                          dither->keycolor,
+                                          output);
+    } else {
+        status = sixel_encode_body(input_pixels,
+                                   width,
+                                   height,
+                                   dither->palette,
+                                   dither->ncolors,
+                                   dither->keycolor,
+                                   dither->bodyonly,
+                                   output,
+                                   NULL,
+                                   dither->allocator);
+    }
+
     if (SIXEL_FAILED(status)) {
         goto end;
     }
